<?php

namespace App\Controller;

use App\Entity\AchatMonnaie;
use App\Entity\AchatMonnaieAConfirmer;
use App\Entity\AchatMonnaieAConfirmerAdherent;
use App\Entity\AchatMonnaieAConfirmerPrestataire;
use App\Entity\AchatMonnaieAdherent;
use App\Entity\AchatMonnaiePrestataire;
use App\Entity\Flux;
use App\Entity\Reconversion;
use App\Events\FluxEvent;
use App\Events\MLCEvents;
use App\Utils\CustomEntityManager;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\IsGranted;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Security;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\StreamedResponse;
use Symfony\Component\Routing\Annotation\Route;

class AdminController extends Controller
{
    private $em;
    private $eventDispatcher;

    public function __construct(CustomEntityManager $em, EventDispatcherInterface $eventDispatcher)
    {
        $this->em = $em;
        $this->eventDispatcher = $eventDispatcher;
    }

    /**
     * @Route("/admin/setenable/{type}/{id}", name="setenable")
     * @IsGranted("ROLE_ADMIN")
     */
    public function setEnableAction($type, $id, Request $request)
    {
        $referer = $request->headers->get('referer');
        if ($referer && !$request->isXmlHttpRequest()) {
            return $this->redirect($referer);
        } elseif (!$request->isXmlHttpRequest()) {
            return new Response('', Response::HTTP_BAD_REQUEST);
        }
        $status = 'success';
        try {
            $object = $this->em->getRepository(str_replace('.', ':', $type))->findOneById($id);
            if ($object && method_exists($object, 'setEnabled')) {
                $object->setEnabled(!$object->isEnabled());
                $this->em->persist($object);
                $this->em->flush();
            } else {
                $status = 'error';
                $this->get('session')->getFlashBag()->add(
                    'notice',
                    'Erreur : veuillez recharger la page et réessayer !'
                );
            }
        } catch (\Exception $e) {
            $status = 'error';
            $this->get('session')->getFlashBag()->add(
                'notice',
                'Erreur : veuillez recharger la page et réessayer !'
            );
        }

        return new JsonResponse(['status' => $status, 'newvalue' => ($object->isEnabled() ? 'true' : 'false')]);
    }

    /**
     * @Route("/admin/setreconverti/{id}", name="setreconverti")
     * @IsGranted({"ROLE_ADMIN_TRANSFERT_GERER_VIEW", "ROLE_ADMIN_RECONVERSION_GERER_ALL"})
     */
    public function setReconvertiAction(Reconversion $object, Request $request)
    {
        $referer = $request->headers->get('referer');
        if ($referer && !$request->isXmlHttpRequest()) {
            return $this->redirect($referer);
        } elseif (!$request->isXmlHttpRequest()) {
            return new Response('', Response::HTTP_BAD_REQUEST);
        }
        $status = 'success';
        try {
            if ($object && method_exists($object, 'setReconverti')) {
                $object->setReconverti(!$object->getReconverti());
                $this->em->persist($object);
                $this->em->flush();
            } else {
                $status = 'error';
            }
        } catch (\Exception $e) {
            $status = 'error';
        }

        return new JsonResponse(['status' => $status, 'newvalue' => ($object->getReconverti() ? 'true' : 'false')]);
    }

    /**
     * @Route("/validate/achat/monnaie/{id}", name="validate_achat_monnaie")
     * @TODO : add is granted role !
     * @Security("user.canValidateAchat == true and (is_granted('ROLE_ADMIN_SIEGE') or is_granted('ROLE_TRESORIER'))")
     */
    public function validateAchatMonnaieAction(Request $request, AchatMonnaieAConfirmer $achatAConfirmer)
    {
        $referer = $request->headers->get('referer');
        if ($referer && !$request->isXmlHttpRequest()) {
            return $this->redirect($referer);
        } elseif (!$request->isXmlHttpRequest()) {
            return new Response('', Response::HTTP_BAD_REQUEST);
        }
        $status = 'success';
        try {
            $flux = new AchatMonnaie();
            if ($achatAConfirmer instanceof AchatMonnaieAConfirmerAdherent) {
                $flux = new AchatMonnaieAdherent();
            } elseif ($achatAConfirmer instanceof AchatMonnaieAConfirmerPrestataire) {
                $flux = new AchatMonnaiePrestataire();
            }
            $flux->setOperateur($this->getUser());
            $flux->setRole($this->getUser()->getGroups()[0]->__toString());
            $flux->setExpediteur($achatAConfirmer->getExpediteur());
            $flux->setDestinataire($achatAConfirmer->getDestinataire());
            $flux->setMontant($achatAConfirmer->getMontant());
            $flux->setMoyen($achatAConfirmer->getMoyen());
            $flux->setReference($achatAConfirmer->getReference());
            $flux->setReconverti(true);
            $flux->setData(['validationAchat' => true]);
            $achatAConfirmer->setReconverti(true);
            $this->em->persist($flux);
            $this->em->persist($achatAConfirmer);
            $this->em->flush();

            // Write operations for this flux !
            $operations = $flux->getAllOperations();
            $em = $this->em;
            $callback = function () use ($em, $operations, $fluxData) {
                foreach ($operations as $operation) {
                    $class = $em->getClassMetadata(get_class($operation->getAccount()));
                    $accounts = $em->getRepository($class->getName());
                    $account = $accounts->find(
                        $operation->getAccount()->getId(),
                        LockMode::PESSIMISTIC_WRITE
                    );
                    $account->addAmount($operation->getMontant());
                    $em->persist($operation);
                    $em->persist($account);
                }
                $em->persist($fluxData);
            };
            $this->em->transactional($callback);

            $this->eventDispatcher->dispatch(
                MLCEvents::VALIDATE_DEMANDE_ACHAT_EMLC,
                new FluxEvent($flux)
            );
        } catch (\Exception $e) {
            // TMP
            throw $e;
            $status = 'error';
        }

        return new JsonResponse(['status' => $status]);
    }

    /**
     * @Route("/admin/getcsv", name="getcsv")
     * @IsGranted({"ROLE_SUPER_ADMIN", "ROLE_ADMIN_IMPORT_EDIT", "ROLE_ADMIN_IMPORT_LIST", "ROLE_ADMIN_IMPORT_CREATE", "ROLE_ADMIN_IMPORT_VIEW", "ROLE_ADMIN_IMPORT_DELETE", "ROLE_ADMIN_IMPORT_EXPORT", "ROLE_ADMIN_IMPORT_ALL"})
     */
    public function getCsvAction(Request $request)
    {
        $data = $request->get('data');
        $header = $request->get('header');
        $response = new StreamedResponse();
        $response->setCallback(function () use ($data, $header) {
            $handle = fopen('php://output', 'w+');
            // Nom des colonnes du CSV
            fputcsv($handle, explode(';', $header), ';');

            //Champs
            foreach ($data as $row) {
                // fwrite($handle, $row);
                fputcsv($handle, explode(';', $row), ';');
            }
            fclose($handle);
        });
        $response->setStatusCode(200);
        $response->headers->set('Content-Type', 'text/csv; charset=utf-8');
        $response->headers->set('Content-Disposition', 'attachment; filename="export.csv"');

        return $response;
    }
}
